//
// (c) 2020 wesolutions GmbH
// All rights reserved.
//

import QtQuick 2.3
import QtGraphicalEffects 1.0

import wesual.Controls 1.0
import wesual.Showcase 1.0

import wesual.data.Core 1.0

FocusScope {
    id : colorScheme

    property list<ColorSchemeItem> colorSchemes

    property IntegerConfiguration colorSchemeIndex
    property int displayColorCount : 4
    property int currentIndex : colorSchemeIndex.value

    readonly property bool open : popup.popupVisible
    readonly property var currentScheme : {
        if (currentIndex >= 0 && currentIndex < colorSchemes.length) {
            return colorSchemes[currentIndex];
        }
        return colorSchemes[0];
    }

    function openPopup()  {
        popup.activeIndex = currentIndex;
        popup.show();
    }

    width : (displayColorCount+1)*22 // colorCount in scheme + indicator
    implicitHeight : 28

    Rectangle {
        id : container

        width : parent.width
        height : 28
        color : UI.color(UI.PrimaryBase)
        border {
            width : 1
            color : (topMouseArea.containsMouse || colorScheme.activeFocus) ?
                   UI.color(UI.PrimaryHover) : UI.color(UI.PrimaryControlBorder)
        }
        Behavior on border.color {
            ColorAnimation { duration : 100 }
        }

        Row {
            anchors {
                top : parent.top
                topMargin : 3
                left : parent.left
                leftMargin : 3
            }

            height : 22
            spacing : 0
            Repeater {
                id : repeater
                model : currentScheme.colors

                delegate : Rectangle {
                    width : 22*displayColorCount/currentScheme.colors.length
                    height : 22
                    color : modelData
                }
            }
        }

        Item {
            id : indicatorContainer
            width : 20
            height : 22
            anchors.right : parent.right

            ColorizedImage {
                id : indicator
                anchors {
                    horizontalCenter : parent.horizontalCenter
                    verticalCenter : parent.verticalCenter
                    verticalCenterOffset : 2
                }
                source : "qrc:/controls/images/dropdown-indicator.png"
                color : (topMouseArea.containsMouse || colorScheme.open
                         || colorScheme.activeFocus) ?
                          UI.color(UI.PrimaryHover) :
                          UI.color(UI.DropdownIndicatorIcon)

            }
        } // indicator wrapper
    }


    MouseArea {
        id : topMouseArea

        hoverEnabled : true
        anchors.fill : parent
        acceptedButtons : Qt.LeftButton
        cursorShape : Qt.PointingHandCursor
        onPressed : colorScheme.open ? popup.hide() : colorScheme.openPopup()
    }

    PopupAnchorItem {
        id : popup

        property var items       : colorScheme.colorSchemes
        property int activeIndex : -1

        signal accepted(int result)

        anchors.centerIn : parent

        onAccepted : {
            colorSchemeIndex.beginRecordChanges();
            colorSchemeIndex.value = result;
            colorSchemeIndex.endRecordChanges();
            popup.hide();
        }

        Keys.onEscapePressed : popup.hide()
        Keys.onUpPressed   : {}
        Keys.onDownPressed : {}
        Keys.onRightPressed : {}
        Keys.onLeftPressed : {}
        Keys.onPressed : {
            if (event.key === Qt.Key_PageUp || event.key === Qt.Key_PageDown) {
                // do not propagate navigation keys while focus item
                event.accepted = true;
            }
        }

        popup : Item {
            id : popupItem

            width : contentWrapper.width
            height : contentWrapper.height

            opacity : 0

            anchors {
                top : popup.popupVisible ? parent.bottom : undefined
                left : popup.popupVisible ? parent.left : undefined
                leftMargin : -popup.parent.width / 2
                topMargin : popup.parent.height / 2
            }

            states : [
                State {
                    name : "visible"
                    when : popup.popupVisible

                    PropertyChanges {
                        target : popupItem
                        opacity : 1
                    }
                }
            ]

            transitions : Transition {
                NumberAnimation {
                    property : "opacity"
                    duration : 200
                }
            }

            Rectangle {
                id : contentWrapper

                readonly property int textWidth : 100

                implicitWidth  : 16*colorScheme.displayColorCount + textWidth
                implicitHeight : content.height + 6

                border {
                    color : UI.color(UI.PrimaryControlBorder)
                    width : 1
                }

                MouseArea { // catch Mouse on background to avoid popup close
                    id : catchArea
                    anchors.fill : parent
                }


                Column {
                    id : content

                    width : parent.width - 2
                    height : childrenRect.height

                    anchors.centerIn : parent

                    Repeater {
                        model : popup.items

                        delegate : Rectangle {
                            id : delegate

                            property ColorSchemeItem colorItem : modelData
                            width : parent.width
                            height : 24

                            color : mouseArea.pressed ? UI.color(UI.PrimaryPress)
                                                      : (mouseArea.containsMouse
                                                         ? UI.color(UI.PrimaryHover)
                                                         : UI.color(UI.PrimaryBase))
                            Behavior on color {
                                ColorAnimation { duration: 200 }
                            }

                            Rectangle {
                                id : colorsWrapper
                                height : 20

                                anchors.verticalCenter : parent.verticalCenter
                                anchors.left : parent.left
                                anchors.leftMargin : 2
                                width : colorsRow.width + 4

                                Row {
                                    id : colorsRow
                                    anchors.verticalCenter : parent.verticalCenter
                                    anchors.left : parent.left
                                    anchors.leftMargin : 2
                                    width : childrenRect.width

                                    Repeater {
                                        id : colorsRepeater

                                        model : modelData.colors
                                        delegate : Rectangle {
                                            anchors {
                                                verticalCenter : parent.verticalCenter
                                            }
                                            width : 16*colorScheme.displayColorCount
                                                    /delegate.colorItem.colors.length
                                            height : 16
                                            color : modelData
                                        }
                                    }
                                }
                            }

                            Text {
                                id : text
                                anchors.left : colorsWrapper.right
                                anchors.leftMargin : 4
                                height : parent.height
                                width : parent.width - colorsWrapper.width - 10
                                text : modelData.text
                                horizontalAlignment : Qt.AlignLeft
                                verticalAlignment : Qt.AlignVCenter
                                font {
                                    family : UI.fontFamily(UI.PrimaryFont)
                                    weight : UI.fontWeight(UI.PrimaryFont)
                                    pixelSize : 13
                                }
                                elide : Qt.ElideRight
                                color : UI.color(UI.PrimaryTextColor)
                            }

                            MouseArea {
                                id : mouseArea
                                anchors.fill : delegate
                                hoverEnabled : true
                                acceptedButtons : Qt.LeftButton
                                onClicked : popup.accepted(index)
                            }
                        } // repeater delegate
                    }
                } // column
            } // contentItem
        }
    } // popup
}
