//
// (c) 2020 wesolutions GmbH
// All rights reserved.
//

import QtQuick 2.2

import wesual.Showcase 1.0
import wesual.Controls 1.0

Item {
    id : editor_

    property ImageConfiguration configuration
    property InlineEditor       editor
    property bool               forceHidden    : false
    property bool               forceFrame     : false
    property Item               mask           : defaultMask

    readonly property bool isActive : editor && editor.isActive && configuration

    Component.onCompleted : editContainer.updateConfiguration()

    onEnabledChanged : {
        if (!enabled) {
             editor.clearActive();
        }
    }
    onIsActiveChanged : {
        if (editor.isActive) {
            editor_.forceActiveFocus();
            slideControl.show();
        } else {
            slideControl.hide();
        }
    }

    Keys.onReturnPressed : editor.clearActive()
    Keys.onEnterPressed  : editor.clearActive()
    Keys.onEscapePressed : editor.clearActive()

    Keys.onPressed: {
        event.accepted = true;

        // change offset
        if (event.key === Qt.Key_Up   || event.key === Qt.Key_Down ||
            event.key === Qt.Key_Left || event.key === Qt.Key_Right) {
            var offset = configuration.offset(mouseArea.size);
            var offsetCorrection = 5;

            if (event.modifiers === Qt.ShiftModifier)
                offsetCorrection = 50;

            configuration.beginRecordChanges();

            switch (event.key) {
                case Qt.Key_Up:
                    offset.y -= offsetCorrection;
                    break;
                case Qt.Key_Down:
                    offset.y += offsetCorrection;
                    break;
                case Qt.Key_Left:
                    offset.x -= offsetCorrection;
                    break;
                case Qt.Key_Right:
                    offset.x += offsetCorrection;
                    break;
            }
            offset = mouseArea.sanitizeOffset(offset);
            configuration.setOffset(
                        Qt.size(editContainer.width, editContainer.height), offset);
            configuration.endRecordChanges();
        } else { // change scaling
            switch (event.key) {
            case Qt.Key_Plus:
                mouseArea.scaleUp();
                break;
            case Qt.Key_Minus:
                mouseArea.scaleDown();
                break;
            case Qt.Key_0:
                if (event.modifiers === Qt.ControlModifier)
                    mouseArea.scaleToFactor(1.0);
                break;
            case Qt.Key_1:
                if (event.modifiers === Qt.ControlModifier)
                    mouseArea.scaleToNativeSize();
                break;
            }
        }
    }

    Rectangle {
        id           : emptyBackgroud
        anchors.fill : parent
        opacity      : (editor_.configuration !== null) ? 0 : 1
        Behavior on opacity {
            NumberAnimation { duration : 150 }
        }
        color  : UI.color(UI.ImageBoxBackground)
        radius : mask && mask["radius"] ? mask.radius : 0
    }

    Item {
        id : editContainer

        anchors.fill : parent

        opacity : (editor_.isActive && image !== null) ? 1.0 : 0.0
        visible : editor.editMode

        property TextureSource image : null

        Connections {
            target               : editContainer.image
            ignoreUnknownSignals : true
            onTextureReady       : shader.image = editContainer.image
        }

        function updateConfiguration() {
            shader.image = null;
            if (editContainer.image) {
                editContainer.image.destroy();
                editContainer.image = null;
            }

            if (editor_.configuration) {
                editContainer.image = preview.createObject(editContainer, {
                    "configuration" : editor_.configuration
                });
            }
        }

        Connections {
            target : editor_
            onConfigurationChanged : editContainer.updateConfiguration()
        }

        Behavior on opacity {
            NumberAnimation { duration : 150 }
        }

        Rectangle {
            id : defaultMask

            visible : false
            width : editor ? editor.width : 1
            height : editor ? editor.height : 1
        }

        Component {
            id : preview

            TextureSource {
                id : image

                property alias configuration : parameters.configuration

                readonly property alias imageWidth  : parameters.imageWidth
                readonly property alias imageHeight : parameters.imageHeight
                readonly property alias offsetX     : parameters.offsetX
                readonly property alias offsetY     : parameters.offsetY
                readonly property alias subrect     : parameters.subrect
                readonly property real  nativeScale : {
                    if (!configuration.asset)
                        return 1.0;

                    var scaleX = editor.width/configuration.asset.size.width;
                    var scaleY = editor.height/configuration.asset.size.height;
                    return scale = 1 / Math.max(scaleX, scaleY);
                }

                mipmap : true
                url    : parameters.image
                requiredSize : configuration.asset ?
                                   configuration.asset.size : Qt.size(1, 1)

                ImageParameters {
                    id : parameters

                    boxWidth  : parseInt(editor ? editor.width : 1)
                    boxHeight : parseInt(editor ? editor.height : 1)
                    onImageChanged   : slideControl.updateScale()
                    onSubrectChanged : slideControl.updateScale()
                }
            }
        }

        ShaderEffectSource {
            id : maskSource

            sourceItem : editor_.mask
            wrapMode   : ShaderEffectSource.ClampToEdge
        }

        ShaderEffect {
            id : shader

            property TextureSource image : null

            blending      : true
            width         : image ? image.imageWidth  : 0
            height        : image ? image.imageHeight : 0
            anchors {
                centerIn               : parent
                verticalCenterOffset   : image ? image.offsetY : 0
                horizontalCenterOffset : image ? image.offsetX : 0
            }
            z       : 1
            opacity : !editor_.forceHidden && (shader.image !== null) ? 1 : 0

            property variant src  : image
            property variant mask : maskSource

            readonly property rect subrect     :
                image ? image.subrect : Qt.rect(0, 0, 1, 1)
            readonly property real clipOpacity : .25

            vertexShader: "
                uniform highp mat4 qt_Matrix;
                attribute highp vec4 qt_Vertex;
                attribute highp vec2 qt_MultiTexCoord0;
                varying highp vec2 coord;
                void main() {
                    coord = qt_MultiTexCoord0;
                    gl_Position = qt_Matrix * qt_Vertex;
                }"

            fragmentShader: "
                varying lowp vec2 coord;
                uniform lowp vec4 subrect;
                uniform sampler2D src;
                uniform sampler2D mask;
                uniform lowp float qt_Opacity;
                uniform lowp float clipOpacity;
                void main() {
                    lowp vec2 maskCoord = vec2(
                        (coord.x - subrect[0]) / subrect[2],
                        (coord.y - subrect[1]) / subrect[3]);
                    lowp vec4 tex = texture2D(src, coord);
                    lowp vec4 m   = texture2D(mask, maskCoord);
                    lowp float ma = m.a *
                        step(subrect[0], coord.x) *
                        step(subrect[1], coord.y) *
                        (1.0 - step(subrect[0] + subrect[2], coord.x)) *
                        (1.0 - step(subrect[1] + subrect[3], coord.y));
                    lowp float finalOpacity =
                        (clipOpacity + (1.0 - clipOpacity) * ma);
                    gl_FragColor = mix(
                        vec4(vec3(dot(tex.rgb, vec3(0.344, 0.5, 0.156))), tex.a),
                        tex, ma) * qt_Opacity * finalOpacity;
                }"
        }

       MouseArea {
            id : mouseArea

            property point startPosition
            property bool  dragActive : false
            property int   startX : 0
            property int   startY : 0

            readonly property alias image : editContainer.image

            readonly property size size :
                Qt.size(parseInt(width), parseInt(height))

            readonly property int maximumOffsetX :
                image ? ((image.imageWidth - editor.width) / 2) : 0
            readonly property int maximumOffsetY :
                image ? (image.imageHeight - editor.height) / 2 : 0

            function scaleUp() {
                scaleByPercent(5);
            }

            function scaleDown() {
                scaleByPercent(-5);
            }

            function scaleByPercent(percent) {
                var scale = configuration.scaleFactor(size) *
                        (1 + percent / 100.0);
                scaleToFactor(scale);
            }

            function scaleToNativeSize() {
                if (!configuration || !image)
                    return;

                var resolution = configuration.asset.size;
                var scale = image.nativeScale;
                if (editor.width >= resolution.width
                        || editor.height >= resolution.height) {
                    scale = 1.0; // don't allow to small textures
                }

                configuration.beginRecordChanges();
                configuration.setScaleFactor(size, scale);
                configuration.setOffset(size, Qt.point(0, 0));
                configuration.endRecordChanges();
            }

            function scaleToFactor(factor) {
                if (!configuration || !image)
                    return;

                configuration.beginRecordChanges();
                configuration.setScaleFactor(size, Math.max(1.0,
                            Math.min(Math.max(image.nativeScale, 2.0),factor)));

                var offset = sanitizeOffset(configuration.offset(size));
                configuration.setOffset(size, offset);
                configuration.endRecordChanges();
            }

            function sanitizeOffset(offset) {
                if (Math.abs(offset.x) > maximumOffsetX) {
                    offset.x = offset.x < 0 ? -maximumOffsetX : maximumOffsetX;
                }

                if (Math.abs(offset.y) > maximumOffsetY) {
                    offset.y = offset.y < 0 ? -maximumOffsetY : maximumOffsetY;
                }

                return offset;
            }

            anchors.fill : parent
            enabled : (editor_.isActive && editor_.configuration)
            cursorShape : {
                if (enabled && configuration) {
                    return Qt.OpenHandCursor;
                }

                return Qt.ArrowCursor;
            }
            onPressed : {
                startPosition = Qt.point(mouse.x, mouse.y);
                dragActive = true;
                mouseArea.preventStealing = true;
                startX = image.offsetX;
                startY = image.offsetY;
                configuration.beginRecordChanges();
            }
            onReleased : {
                dragActive = false
                configuration.endRecordChanges();
                slideControl.show();
            }
            onWheel : {
                if (wheel.angleDelta.y < 0) {
                    scaleDown();
                } else {
                    scaleUp();
                }
            }
            onPositionChanged : {
                if (mouse.buttons !== Qt.LeftButton)
                    return;

                var offsetX = mouse.x - startPosition.x + startX;
                var offsetY = mouse.y - startPosition.y + startY;

                var offset = Qt.point(Math.round(offsetX), Math.round(offsetY));
                offset = sanitizeOffset(offset);

                configuration.setOffset(Qt.size(width, height), offset);
            }
        }
    }

    ImageScaleSlider {
       id : slideControl

       attachedItem : editor_
       maxValue : editContainer.image
                  ? Math.max(2.0, editContainer.image.nativeScale) : 2.0
       onScaleChanged : {
           mouseArea.scaleToFactor(scale);
       }

       function updateScale() {
           var scale = configuration ?
                       configuration.scaleFactor(mouseArea.size) : 1.0;
           setScale(scale);
       }
    }

    Frame {
        anchors.fill : parent
        frameWidth : 2

        opacity : {
            if (editor.isActive) {
                return 0.8;
            } else if (editor.containsMouse || editor_.forceFrame) {
                return 0.5;
            }
            return 0.0;
        }

        Behavior on opacity {
            NumberAnimation { duration : 150 }
        }
    }
}
