//
// (c) 2020 wesolutions GmbH
// All rights reserved.
//

import QtQuick 2.12

import wesual.Ui 1.0

import wesual.Ui.Private 1.0

FocusScope {
    id : uiDateEdit

    signal editingFinished()

    property date date
    property date minimumDate
    property date maximumDate

    property alias font  : label.font
    property alias color : label.color

    property int   horizontalMargin : 6
    property int   verticalMargin   : 4

    property bool  bindable : false

    readonly property bool hover : mouseArea.containsMouse

    signal updateRequested(date newValue)

    implicitWidth    : 112
    implicitHeight   : label.implicitHeight + 9
    activeFocusOnTab : true
    baselineOffset   : label.y + label.baselineOffset

    onActiveFocusChanged : {
        if (activeFocus) {
            p_.beginEdit();
        } else {
            p_.endEdit();
        }
    }

    Keys.onEnterPressed  : p_.endEdit()
    Keys.onReturnPressed : p_.endEdit()

    QtObject {
        id : p_

        property TextInput editor : null

        readonly property var fullFormats : [
            "d.M.yyyy",
            "yyyy-M-d",
            "M/d/yyyy",
        ]

        readonly property var shortFormats : [
            "d.M.yy",
            "yy-M-d",
            "M/d/yy"
        ]

        function beginEdit() {
            if (editor) {
                editor.forceActiveFocus();
                editor.selectAll();
            } else {
                editor = lineEdit.createObject(uiDateEdit);
            }
        }

        function endEdit() {
            if (!editor)
                return;

            commitEdit();
            editor.destroy();
            uiDateEdit.editingFinished();
            uiDateEdit.focus = false;
        }

        function cancelEdit() {
            if (!editor)
                return;

            editor.destroy();
            editor = null;
            uiDateEdit.focus = false;
        }

        function commitEdit() {
            if (!editor)
                return;

            var date;
            var locale = Qt.locale();
            for (var i = 0; i < fullFormats.length; ++i) {
                date = Date.fromLocaleDateString(
                            locale, editor.text, fullFormats[i]);
                if (date.getFullYear() > 0)
                    break;
            }

            if (date.getFullYear() > 0) {
                setDate(date);
            }

            for (i = 0; i < shortFormats.length; ++i) {
                date = Date.fromLocaleDateString(
                            locale, editor.text, shortFormats[i]);
                if (date.getFullYear() > 0)
                    break;
            }

            if (date.getFullYear() > 0) {
                // Use current century for short date formats
                var century = new Date().getFullYear();
                century -= century % 100;
                date.setFullYear(century + date.getFullYear() % 100);

                setDate(date);
            }
        }

        function setDate(date) {
            date.setHours(0);
            date.setMinutes(0);
            date.setSeconds(0);
            date.setMilliseconds(0);

            if (uiDateEdit.bindable) {
                uiDateEdit.updateRequested(date);
            } else {
                uiDateEdit.date = date;
            }
        }

        function formatDate(date) {
            var locale = Qt.locale();
            var format = locale.dateFormat(Locale.ShortFormat);
            format = format.replace(/d+|M+|y+/g, function(match, index) {
                if (match === "d") {
                    return "dd"
                } else if (match === "M") {
                    return "MM"
                } else if (match === "yy") {
                    return "yyyy";
                }

                return match;
            });

            return date.toLocaleDateString(locale, format);
        }
    }

    UiToolButton {
        id : button

        anchors {
            right  : parent.right
            top    : parent.top
            bottom : parent.bottom
        }
        width     : height - 1
        iconName  : "calendar"
        z         : 100
        onClicked : popup.show()
        down      : popup.popupVisible
    }

    UiPopupAnchor {
        id : popup

        anchors.left   : parent.left
        anchors.bottom : parent.bottom
        triggerItem    : button
        autoLayout     : true
        onAboutToShow  : uiDateEdit.forceActiveFocus()

        popup : UiCalendar {
            id : calendar

            opacity  : 0
            enabled  : false
            y        : -1
            bindable : true

            date        : uiDateEdit.date
            minimumDate : uiDateEdit.minimumDate
            maximumDate : uiDateEdit.maximumDate

            onUpdateRequested : {
                p_.setDate(newValue);
                popup.hide();
            }

            states : State {
                name : "visible"
                when : popup.popupVisible

                PropertyChanges {
                    target  : calendar
                    opacity : 1
                    enabled : true
                }
            }

            transitions : Transition {
                NumberAnimation {
                    property : "opacity"
                    duration : 100
                }
            }
        }
    }

    Rectangle {
        id : background

        anchors.fill : parent
        color : UiColors.getColor(UiColors.White)
        border {
            width : 1
            color : UiColors.getColor(UiColors.ControlGrey20)
        }
        z : -1

        states : [
            State {
                name : "disabled"
                when : !uiDateEdit.enabled

                PropertyChanges {
                    target       : background
                    border.color : UiColors.getColor(
                                       UiColors.DisabledLightGrey13)
                }
            },
            State {
                name : "focus"
                when : uiDateEdit.activeFocus

                PropertyChanges {
                    target       : background
                    border.color : UiColors.getColor(UiColors.SelectionGreen)
                }
            },
            State {
                name : "hover"
                when : uiDateEdit.hover

                PropertyChanges {
                    target       : background
                    border.color : UiColors.getColor(UiColors.HoverGreen)
                }
            }
        ]
    }

    UiTextContextMenu {
        textEditor             : p_.editor
        onContextMenuRequested : uiDateEdit.forceActiveFocus()
    }

    MouseArea {
        id : mouseArea

        anchors.fill : parent
        onClicked    : uiDateEdit.forceActiveFocus()
        hoverEnabled : true
        cursorShape  : Qt.IBeamCursor
    }

    Text {
        id : label

        x        : uiDateEdit.horizontalMargin
        y        : uiDateEdit.verticalMargin
        font     : UiFonts.getFont(UiFonts.Light, 14)
        color    : UiColors.getColor(UiColors.Black)
        width    : uiDateEdit.width - 2 * uiDateEdit.horizontalMargin + 1
        elide    : Text.ElideRight
        wrapMode : Text.NoWrap
        opacity  : p_.editor ? 0 : 1
        text     : {
            var now = new Date();
            var today = uiDateEdit.date.getFullYear() === now.getFullYear() &&
                    uiDateEdit.date.getMonth() === now.getMonth() &&
                    uiDateEdit.date.getDate() === now.getDate();

            if (today) {
                return qsTrId("button.Calendar.Today");
            }

            return p_.formatDate(uiDateEdit.date);
        }

        states : [
            State {
                name : "disabled"
                when : !uiDateEdit.enabled

                PropertyChanges {
                    target : label
                    color  : UiColors.getColor(UiColors.MediumLightGrey27)
                }
            }
        ]
    }

    Component {
        id : lineEdit

        TextInput {
            id : editor

            x      : label.x
            y      : label.y
            width  : uiDateEdit.width - 2 * uiDateEdit.horizontalMargin
            height : label.implicitHeight
            font   : label.font
            color  : label.color

            selectByMouse     : true
            selectionColor    : UiColors.getColor(UiColors.SelectionGreen)
            selectedTextColor : UiColors.getColor(UiColors.White)
            clip              : true

            Component.onCompleted : {
                text = p_.formatDate(uiDateEdit.date);
                forceActiveFocus();
                selectAll();
            }

            Keys.onEscapePressed : p_.cancelEdit()

            Connections {
                target        : uiDateEdit
                onDateChanged : {
                    editor.text = p_.formatDate(uiDateEdit.date);
                    editor.selectAll();
                }
            }
        }
    }
}

