//
// (c) 2020 wesolutions GmbH
// All rights reserved.
//

import QtQuick 2.12

import wesual.Ui 1.0

import wesual.Ui.Private 1.0

FocusScope {
    id : uiTimeEdit

    signal editingFinished()

    property date time
    property date minimumTime
    property date maximumTime

    property alias font  : label.font
    property alias color : label.color

    property int   horizontalMargin : 6
    property int   verticalMargin   : 4

    property bool  bindable : false

    readonly property bool hover : mouseArea.containsMouse

    signal updateRequested(date newValue)

    implicitWidth    : 48
    implicitHeight   : label.implicitHeight + 9
    activeFocusOnTab : true
    baselineOffset   : label.y + label.baselineOffset

    onActiveFocusChanged : {
        if (activeFocus) {
            p_.beginEdit();
        } else {
            p_.endEdit();
        }
    }

    Keys.onEnterPressed  : p_.endEdit()
    Keys.onReturnPressed : p_.endEdit()

    QtObject {
        id : p_

        property TextInput editor : null

        function beginEdit() {
            if (editor) {
                editor.forceActiveFocus();
                editor.selectAll();
            } else {
                editor = lineEdit.createObject(uiTimeEdit);
            }
        }

        function endEdit() {
            if (!editor)
                return;

            commitEdit();
            editor.destroy();
            uiTimeEdit.editingFinished();
            uiTimeEdit.focus = false;
        }

        function cancelEdit() {
            if (!editor)
                return;

            editor.destroy();
            editor = null;
            uiTimeEdit.focus = false;
        }

        function commitEdit() {
            if (!editor)
                return;

            var locale = Qt.locale();
            var time = editor.text;

            if (/^\d{1,2}:$/.test(time)) {
                time += "00";
            }

            var time = Date.fromLocaleTimeString(locale, time, "H:m");
            if (time.getFullYear() > 0)
                setTime(time);
        }

        function setTime(time) {
            if (uiTimeEdit.bindable) {
                uiTimeEdit.updateRequested(time);
            } else {
                uiTimeEdit.time = time;
            }
        }
    }

    Rectangle {
        id : background

        anchors.fill : parent
        color : UiColors.getColor(UiColors.White)
        border {
            width : 1
            color : UiColors.getColor(UiColors.ControlGrey20)
        }
        z : -1

        states : [
            State {
                name : "disabled"
                when : !uiTimeEdit.enabled

                PropertyChanges {
                    target       : background
                    border.color : UiColors.getColor(
                                       UiColors.DisabledLightGrey13)
                }
            },
            State {
                name : "focus"
                when : uiTimeEdit.activeFocus

                PropertyChanges {
                    target       : background
                    border.color : UiColors.getColor(UiColors.SelectionGreen)
                }
            },
            State {
                name : "hover"
                when : uiTimeEdit.hover

                PropertyChanges {
                    target       : background
                    border.color : UiColors.getColor(UiColors.HoverGreen)
                }
            }
        ]
    }

    UiTextContextMenu {
        textEditor             : p_.editor
        onContextMenuRequested : uiTimeEdit.forceActiveFocus()
    }

    MouseArea {
        id : mouseArea

        anchors.fill : parent
        onClicked    : uiTimeEdit.forceActiveFocus()
        hoverEnabled : true
        cursorShape  : Qt.IBeamCursor
    }

    Text {
        id : label

        x        : uiTimeEdit.horizontalMargin
        y        : uiTimeEdit.verticalMargin
        font     : UiFonts.getFont(UiFonts.Light, 14)
        color    : UiColors.getColor(UiColors.Black)
        width    : uiTimeEdit.width - 2 * uiTimeEdit.horizontalMargin + 1
        elide    : Text.ElideRight
        wrapMode : Text.NoWrap
        opacity  : p_.editor ? 0 : 1
        text     : uiTimeEdit.time.toLocaleString(Qt.locale(), "HH:mm")

        states : [
            State {
                name : "disabled"
                when : !uiTimeEdit.enabled

                PropertyChanges {
                    target : label
                    color  : UiColors.getColor(UiColors.MediumLightGrey27)
                }
            }
        ]
    }

    Component {
        id : lineEdit

        TextInput {
            x      : label.x
            y      : label.y
            width  : uiTimeEdit.width - 2 * uiTimeEdit.horizontalMargin
            height : label.implicitHeight
            font   : label.font
            color  : label.color

            selectByMouse     : true
            selectionColor    : UiColors.getColor(UiColors.SelectionGreen)
            selectedTextColor : UiColors.getColor(UiColors.White)
            clip              : true
            inputMask         : "00:00"

            Component.onCompleted : {
                text = label.text;
                forceActiveFocus();
                selectAll();
            }

            Keys.onEscapePressed : p_.cancelEdit()

            Connections {
                target        : uiTimeEdit
                onTimeChanged : text = label.text
            }
        }
    }
}

