//
// (c) 2020 wesolutions GmbH
// All rights reserved.
//

import QtQuick 2.12

import wesual.Ui 1.0

/*!
\qmltype     UiTextEditBase
\inqmlmodule we.captivo.Ui.Private
\inherits    QtQuick2::FocusScope
\qmlabstract
\internal

\brief The base class for all text edit types.
*/

FocusScope {
    id : uiTextEdit

    /*!
    \brief Emitted when a text edit operation is finished.

    Text editing finishes when the control loses focus or the user presses
    the escape key during editing.
    */
    signal editingFinished()

    /*!
    \brief The editable text.

    \nobinding
    */
    property string text

    /*!
    \brief The placeholder text.

    The placeholder text is displayed in place of an empty text when the
    control does not have edit focus.
    */
    property string placeHolderText

    /*!
    \brief The horizontal margin between text and element border
    \internal
    */
    property int horizontalMargin : 0

    /*!
    \brief The vertical margin between text and element border
    \internal
    */
    property int verticalMargin   : 0

    /*!
    \brief Set to \c true while the controls is hovered by the mouse pointer.
    */
    readonly property bool hover : mouseArea.containsMouse

    /*!
    \brief The minimum number of text lines to display.

    The control will limit its \l {Item::}{implicitHeight} to be large enough
    to display the given number of lines.

    \note
    Setting the height of the control manually will disable the effect of
    this property.

    \sa maximumDisplayLines
    */
    property int minimumDisplayLines : 2

    /*!
    \brief The maximum number of text lines to display.

    The control will limits its \l {Item::}{implicitHeight} to display no more
    than the given number of lines.

    \note
    Setting the height of the control manually will disable the effect of
    this property.

    \sa minimumDisplayLines
    */
    property int maximumDisplayLines : 10

    property bool bindable : false

    signal updateRequested(string newValue)

    implicitWidth    : 250
    implicitHeight   : p_.implicitHeight
    activeFocusOnTab : true
    baselineOffset   : label.baselineOffset + verticalMargin

    onActiveFocusChanged : {
        if (activeFocus) {
            p_.beginEdit();
        } else {
            p_.endEdit();
        }
    }


    QtObject {
        id : p_

        property TextEdit editor : null;

        readonly property int implicitHeight : {
            var lineHeight = metrics.height;
            var min = lineHeight * minimumDisplayLines;
            var max = lineHeight * maximumDisplayLines;
            var iw = editor ? editor.implicitHeight : label.implicitHeight;

            iw = Math.max(min, Math.min(max, iw));

            iw += 2 * uiTextEdit.verticalMargin;

            return iw;
        }

        function beginEdit() {
            if (editor) {
                editor.forceActiveFocus();
                editor.selectAll();
            } else {
                editor = textEdit.createObject(mouseArea);
            }
        }

        function endEdit() {
            if (!editor)
                return;

            commitEdit();
            editor.destroy();
            editor = null;
            uiTextEdit.editingFinished();
            uiTextEdit.focus = false;
        }

        function cancelEdit() {
            if (!editor)
                return;

            editor.destroy();
            editor = null;
            uiTextEdit.focus = false;
        }

        function commitEdit() {
            if (!editor)
                return;

            if (uiTextEdit.bindable) {
                uiTextEdit.updateRequested(editor.text);
            } else {
                uiTextEdit.text = editor.text;
            }
        }
    }

    UiTextContextMenu {
        textEditor             : p_.editor
        onContextMenuRequested : uiTextEdit.forceActiveFocus()
    }

    MouseArea {
        id : mouseArea

        anchors.fill : parent
        onClicked    : uiTextEdit.forceActiveFocus()
        hoverEnabled : true
        clip         : true
        cursorShape  : Qt.IBeamCursor
    }

    FontMetrics {
        id : metrics

        font : label.font
    }

    Text {
        id : label

        x        : uiTextEdit.horizontalMargin
        y        : uiTextEdit.verticalMargin
        font     : UiFonts.getFont(UiFonts.Light, 14)
        color    : UiColors.getColor(UiColors.Black)
        width    : uiTextEdit.width - 2 * uiTextEdit.horizontalMargin + 1
        elide    : Text.ElideRight
        wrapMode : Text.WordWrap
        opacity  : p_.editor ? 0 : 1
        text     : uiTextEdit.text

        maximumLineCount : uiTextEdit.maximumDisplayLines

        states : [
            State {
                name : "disabled"

                PropertyChanges {
                    target : label
                    color  : UiColors.getColor(UiColors.MediumLightGrey27)
                }
            },
            State {
                name : "empty"
                when : uiTextEdit.text.length === 0

                PropertyChanges {
                    target : label
                    text   : uiTextEdit.placeHolderText
                    color  : UiColors.getColor(UiColors.MediumGrey33)
                }
            }
        ]
    }

    Component {
        id : textEdit

        TextEdit {
            readonly property int offset : {
                return Math.max(cursorRectangle.y +
                                cursorRectangle.height - height, 0);
            }

            x      : label.x
            y      : label.y - offset
            width  : uiTextEdit.width - 2 * uiTextEdit.horizontalMargin
            height : uiTextEdit.height - 2 * uiTextEdit.verticalMargin
            font   : label.font
            color  : UiColors.getColor(UiColors.Black)

            wrapMode          : Text.WordWrap
            selectByMouse     : true
            selectionColor    : UiColors.getColor(UiColors.SelectionGreen)
            selectedTextColor : UiColors.getColor(UiColors.White)

            Component.onCompleted : {
                text = uiTextEdit.text;
                forceActiveFocus();
                cursorPosition = text.length;
                selectAll();
            }
            Keys.onTabPressed : {
                var next = nextItemInFocusChain(true);
                if (next) {
                    next.forceActiveFocus();
                }
            }
            Keys.onEscapePressed : p_.cancelEdit()
        }
    }
}

